import { AbstractPowerSyncDatabaseOpenFactory } from '@journeyapps/powersync-sdk-common';
import { PowerSyncDatabase } from '../../db/PowerSyncDatabase';
import { SSRDBAdapter } from './SSRDBAdapter';
export const DEFAULT_POWERSYNC_FLAGS = {
    /**
     * Multiple tabs are by default not supported on Android, iOS and Safari.
     * Other platforms will have multiple tabs enabled by default.
     */
    enableMultiTabs: typeof globalThis.navigator !== 'undefined' && // For SSR purposes
        typeof SharedWorker !== 'undefined' &&
        !navigator.userAgent.match(/(Android|iPhone|iPod|iPad)/i) &&
        !window.safari
};
/**
 * Intermediate PowerSync Database Open factory for Web which uses a mock
 * SSR DB Adapter if running on server side.
 * Most SQLite DB implementations only run on client side, this will safely return
 * empty query results in SSR which will allow for generating server partial views.
 */
export class AbstractWebPowerSyncDatabaseOpenFactory extends AbstractPowerSyncDatabaseOpenFactory {
    isServerSide() {
        return typeof window == 'undefined';
    }
    constructor(options) {
        super(options);
        this.options = options;
    }
    generateOptions() {
        var _a;
        const isServerSide = this.isServerSide();
        if (isServerSide && !((_a = this.options.flags) === null || _a === void 0 ? void 0 : _a.disableSSRWarning)) {
            console.warn(`
  Running PowerSync in SSR mode.
  Only empty query results will be returned.
  Disable this warning by setting 'disableSSRWarning: true' in options.`);
        }
        // Resolve flags for PowerSync DB client
        const resolvedFlags = this.resolveDBFlags();
        if (!resolvedFlags.enableMultiTabs) {
            console.warn('Multiple tab support is not enabled. Using this site across multiple tabs may not function correctly.');
        }
        return Object.assign(Object.assign({}, this.options), { database: isServerSide ? new SSRDBAdapter() : this.openDB(), schema: this.schema, flags: resolvedFlags });
    }
    resolveDBFlags() {
        var _a;
        let flags = Object.assign(Object.assign({}, DEFAULT_POWERSYNC_FLAGS), { ssrMode: this.isServerSide() });
        if (typeof ((_a = this.options.flags) === null || _a === void 0 ? void 0 : _a.enableMultiTabs) != 'undefined') {
            flags.enableMultiTabs = this.options.flags.enableMultiTabs;
        }
        return flags;
    }
    generateInstance(options) {
        return new PowerSyncDatabase(options);
    }
}
//# sourceMappingURL=AbstractWebPowerSyncDatabaseOpenFactory.js.map